#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Overlay window for voice"""
import logging
import math
import cairo
import sys
from .overlay import OverlayWindow
from .image_getter import get_surface, draw_img_to_rect
# pylint: disable=wrong-import-order
import gi
gi.require_version("Gtk", "3.0")
gi.require_version('PangoCairo', '1.0')
# pylint: disable=wrong-import-position,wrong-import-order
from gi.repository import Pango, PangoCairo  # nopep8
import random

log = logging.getLogger(__name__)


class VoiceOverlayWindow(OverlayWindow):
    """Overlay window for voice"""

    def __init__(self, discover, piggyback=None):
        OverlayWindow.__init__(self, discover, piggyback)

        self.avatars = {}

        self.dummy_data = []
        mostly_false = [False, False, False, False, False, False, False, True]
        for i in range(0,100):
            speaking = mostly_false[random.randint(0,7)]
            scream = ''
            if random.randint(0,20)==2:
                scream = random.randint(8,15)*'a'
            name = "Player %d %s" % (i, scream)
            self.dummy_data.append({
                "id": i,
                "username": name,
                "avatar": None,
                "mute": False,
                "deaf": mostly_false[random.randint(0,7)],
                "mute": mostly_false[random.randint(0,7)],
                "speaking": speaking ,
                'lastspoken': random.randint(2000,2100) if speaking else random.randint(10,30),
                'friendlyname': name,
            })
        self.avatar_size = 48
        self.text_pad = 6
        self.text_font = None
        self.text_size = 13
        self.text_baseline_adj = 0
        self.icon_spacing = 8
        self.vert_edge_padding = 0
        self.horz_edge_padding = 0
        self.only_speaking = None
        self.highlight_self = None
        self.order = None
        self.def_avatar = None
        self.overflow = None
        self.use_dummy = False
        self.dummy_count = 10

        self.round_avatar = True
        self.icon_only = True
        self.talk_col = [0.0, 0.6, 0.0, 0.1]
        self.text_col = [1.0, 1.0, 1.0, 1.0]
        self.text_hili_col = [1.0, 1.0, 1.0, 1.0]
        self.norm_col = [0.0, 0.0, 0.0, 0.5]
        self.wind_col = [0.0, 0.0, 0.0, 0.0]
        self.mute_col = [0.7, 0.0, 0.0, 1.0]
        self.hili_col = [0.0, 0.0, 0.0, 0.9]
        self.border_col = [0.0, 0.0, 0.0, 0.0]
        self.userlist = []
        self.connected = False
        self.horizontal = False
        self.guild_ids = tuple()
        self.force_location()
        get_surface(self.recv_avatar,
                    "share/icons/hicolor/256x256/apps/discover-overlay-default.png",
                    'def', self.avatar_size)
        self.set_title("Discover Voice")
        self.redraw()

    def set_show_dummy(self, show_dummy):
        """
        Toggle use of dummy userdata to help choose settings
        """
        self.use_dummy = show_dummy
        self.needsredraw = True

    def set_dummy_count(self, dummy_count):
        self.dummy_count = dummy_count
        self.needsredraw = True

    def set_overflow(self, overflow):
        """
        How should excessive numbers of users be dealt with?
        """
        self.overflow = overflow
        self.needsredraw = True

    def set_bg(self, background_colour):
        """
        Set the background colour
        """
        self.norm_col = background_colour
        self.needsredraw = True

    def set_fg(self, foreground_colour):
        """
        Set the text colour
        """
        self.text_col = foreground_colour
        self.needsredraw = True

    def set_tk(self, talking_colour):
        """
        Set the border colour for users who are talking
        """
        self.talk_col = talking_colour
        self.needsredraw = True

    def set_mt(self, mute_colour):
        """
        Set the colour of mute and deafen logos
        """
        self.mute_col = mute_colour
        self.needsredraw = True

    def set_hi(self, highlight_colour):
        """
        Set the colour of background for speaking users
        """
        self.hili_col = highlight_colour
        self.needsredraw = True

    def set_fg_hi(self, highlight_colour):
        """
        Set the colour of background for speaking users
        """
        self.text_hili_col = highlight_colour
        self.needsredraw = True

    def set_bo(self, border_colour):
        """
        Set the colour for idle border
        """
        self.border_col = border_colour
        self.needsredraw = True

    def set_avatar_size(self, size):
        """
        Set the size of the avatar icons
        """
        self.avatar_size = size
        self.needsredraw = True

    def set_icon_spacing(self, i):
        """
        Set the spacing between avatar icons
        """
        self.icon_spacing = i
        self.needsredraw = True

    def set_text_padding(self, i):
        """
        Set padding between text and border
        """
        self.text_pad = i
        self.needsredraw = True

    def set_text_baseline_adj(self, i):
        """
        Set padding between text and border
        """
        self.text_baseline_adj = i
        self.needsredraw = True

    def set_vert_edge_padding(self, i):
        """
        Set padding between top/bottom of screen and overlay contents
        """
        self.vert_edge_padding = i
        self.needsredraw = True

    def set_horz_edge_padding(self, i):
        """
        Set padding between left/right of screen and overlay contents
        """
        self.horz_edge_padding = i
        self.needsredraw = True

    def set_square_avatar(self, i):
        """
        Set if the overlay should crop avatars to a circle or show full square image
        """
        self.round_avatar = not i
        self.needsredraw = True

    def set_only_speaking(self, only_speaking):
        """
        Set if overlay should only show people who are talking
        """
        self.only_speaking = only_speaking

    def set_highlight_self(self, highlight_self):
        """
        Set if the overlay should highlight the user
        """
        self.highlight_self = highlight_self

    def set_order(self, i):
        """
        Set the method used to order avatar icons & names
        """
        self.order = i
        self.sort_list(self.userlist)
        self.needsredraw= True

    def set_icon_only(self, i):
        """
        Set if the overlay should draw only the icon
        """
        self.icon_only = i
        self.needsredraw = True

    def set_horizontal(self, horizontal=False):
        self.horizontal = horizontal
        self.needsredraw = True

    def set_guild_ids(self, guild_ids=tuple()):
        if self.discover.connection:
            for _id in guild_ids:
                if _id not in self.guild_ids:
                    self.discover.connection.req_channels(_id)
        self.guild_ids = guild_ids

    def set_wind_col(self):
        """
        Use window colour to draw
        """
        self.col(self.wind_col)

    def set_norm_col(self):
        """
        Use background colour to draw
        """
        self.col(self.norm_col)

    def set_talk_col(self, alpha=1.0):
        """
        Use talking colour to draw
        """
        self.col(self.talk_col, alpha)

    def set_mute_col(self, alpha=1.0):
        """
        Use mute colour to draw
        """
        self.col(self.mute_col, alpha)

    def set_user_list(self, userlist, alt):
        """
        Set the users in list to draw
        """
        self.userlist = userlist
        for user in userlist:
            if "nick" in user:
                user["friendlyname"] = user["nick"]
            else:
                user["friendlyname"] = user["username"]
        self.sort_list(self.userlist)
        screen = self.get_screen()
        # Check if composite state has changed
        if not self.compositing == screen.is_composited():
            alt = True
            self.compositing = screen.is_composited()
        if alt:
            self.needsredraw = True

    def set_connection(self, connection):
        """
        Set if discord has a clean connection to server
        """
        is_connected = connection == "VOICE_CONNECTED"
        if self.connected != is_connected:
            self.connected = is_connected
            self.needsredraw = True

    def sort_list(self, in_list):
        if self.order == 1:  # ID Sort
            in_list.sort(key=lambda x: x["id"])
        elif self.order == 2:  # Spoken sort
            in_list.sort(key=lambda x: x["lastspoken"], reverse=True)
            in_list.sort(key=lambda x: x["speaking"], reverse=True)
        else:  # Name sort
            in_list.sort(key=lambda x: x["friendlyname"])
        return in_list

    def overlay_draw(self, w, context, data=None):
        """
        Draw the Overlay
        """
        self.context = context
        context.set_antialias(cairo.ANTIALIAS_GOOD)
        # Get size of window
        (width, height) = self.get_size()
        # Make background transparent
        self.set_wind_col()
        # Don't layer drawing over each other, always replace
        context.set_operator(cairo.OPERATOR_SOURCE)
        context.paint()
        context.save()
        if self.piggyback:
            self.piggyback.overlay_draw(w, context, data)
        if self.is_wayland or self.piggyback_parent or self.discover.steamos:
            # Special case!
            # The window is full-screen regardless of what the user has selected.
            # We need to set a clip and a transform to imitate original behaviour
            # Used in wlroots & gamescope
            width = self.width
            height = self.height
            if self.floating:
                context.translate(self.pos_x, self.pos_y)
                context.rectangle(0, 0, width, height)
                context.clip()

        context.set_operator(cairo.OPERATOR_OVER)
        if not self.connected and not self.use_dummy:
            return

        connection = self.discover.connection
        self_user = connection.user

        # Gather which users to draw
        users_to_draw = self.userlist[:]
        userlist = self.userlist
        if self.use_dummy: # Sorting every frame is an awful idea. Maybe put this off elsewhere?
            users_to_draw = self.sort_list(self.dummy_data[0:self.dummy_count])
            userlist = self.dummy_data
        for user in userlist:
            # Bad object equality here, so we need to reassign
            if user["id"] == self_user["id"]:
                self_user = user

            # Update friendly name with nick if possible
            if "nick" in user:
                user["friendlyname"] = user["nick"]
            else:
                user["friendlyname"] = user["username"]

            # Remove users that arent speaking
            if self.only_speaking:
                speaking = "speaking" in user and user["speaking"]
                if not speaking:
                    if user in users_to_draw: 
                        users_to_draw.remove(user)

        if self.highlight_self:
            if self_user in users_to_draw:
                users_to_draw.remove(self_user)
            users_to_draw.insert(0, self_user)

        avatar_size = self.avatar_size
        avatars_per_row = sys.maxsize

        if self.horizontal:
            needed_width = (len(users_to_draw) * self.avatar_size) + \
                (len(users_to_draw) + 1) * self.icon_spacing

            if needed_width > width:
                if self.overflow == 1: # Wrap
                    avatars_per_row = int(width / (avatar_size+self.icon_spacing))
                elif self.overflow == 2: # Shrink
                    available_size = width / len(users_to_draw)
                    avatar_size = available_size - self.icon_spacing
                    if avatar_size<8:
                        avatar_size = 8

            current_y = 0 + self.vert_edge_padding
            offset_y = avatar_size + self.icon_spacing
            if self.align_right: # A lie. Align bottom
                current_y = self.height - avatar_size - self.vert_edge_padding
                offset_y = -(avatar_size + self.icon_spacing)
            rows_to_draw = []
            while len(users_to_draw) > 0:
                row = []
                for i in range(0, min(avatars_per_row, len(users_to_draw))):
                    row.append(users_to_draw.pop(0))
                rows_to_draw.append(row)
            for row in rows_to_draw:
                needed_width = (len(row) * (avatar_size + self.icon_spacing))
                current_x = 0 + self.horz_edge_padding
                if self.align_vert == 1:
                    current_x = (width / 2) - (needed_width) / 2
                elif self.align_vert == 2:
                    current_x = width - needed_width - self.horz_edge_padding

                for user in row:
                    self.draw_avatar(context, user, current_x, current_y, avatar_size)
                    current_x += avatar_size + self.icon_spacing
                current_y+=offset_y
        else:
            # Calculate height needed to show overlay
            needed_height = (len(users_to_draw) * self.avatar_size) + \
                (len(users_to_draw) + 1) * self.icon_spacing

            if needed_height > height:
                if self.overflow == 1: # Wrap
                    avatars_per_row = int(height / (avatar_size + self.icon_spacing))
                elif self.overflow == 2: # Shrink
                    available_size = height / len(users_to_draw)
                    avatar_size = available_size - self.icon_spacing
                    if avatar_size<8:
                        avatar_size = 8

            current_x = 0
            offset_x_mult=1
            offset_x = avatar_size + self.horz_edge_padding
            if self.align_right:
                offset_x_mult =-1
                current_x = self.width - avatar_size - self.horz_edge_padding

            # Choose where to start drawing
            current_y = 0 + self.vert_edge_padding
            if self.align_vert == 1:
                current_y = (height / 2) - (needed_height / 2)
            elif self.align_vert == 2:
                current_y = height - needed_height - self.vert_edge_padding

            cols_to_draw = []
            while len(users_to_draw) > 0:
                col = []
                for i in range(0,min(avatars_per_row, len(users_to_draw))):
                    col.append(users_to_draw.pop(0))
                cols_to_draw.append(col)
            for col in cols_to_draw:
                needed_height = (len(col) * (avatar_size + self.icon_spacing))
                current_y = 0 + self.vert_edge_padding
                if self.align_vert == 1:
                    current_y = (height/2) - (needed_height /2)
                elif self.align_vert == 2:
                    current_y = height - needed_height - self.vert_edge_padding
                largest_text_width = 0
                for user in col:
                    text_width = self.draw_avatar(context, user, current_x, current_y, avatar_size)
                    largest_text_width = max(text_width, largest_text_width)
                    current_y += avatar_size + self.icon_spacing
                if largest_text_width > 0:
                    largest_text_width += self.text_pad
                else:
                    largest_text_width = self.icon_spacing
                current_x += offset_x_mult* (offset_x + largest_text_width)

        context.restore()
        self.context = None

    def recv_avatar(self, identifier, pix):
        """
        Called when image_getter has downloaded an image
        """
        if identifier == 'def':
            self.def_avatar = pix
        else:
            self.avatars[identifier] = pix
        self.needsredraw = True

    def delete_avatar(self, identifier):
        """
        Remove avatar image
        """
        if identifier in self.avatars:
            del self.avatars[identifier]

    def draw_avatar(self, context, user, pos_x, pos_y, avatar_size):
        """
        Draw avatar at given Y position. Includes both text and image based on settings
        """
        # Ensure pixbuf for avatar
        if user["id"] not in self.avatars and user["avatar"]:
            url = "https://cdn.discordapp.com/avatars/%s/%s.png" % (
                user['id'], user['avatar'])
            get_surface(self.recv_avatar, url, user["id"],
                        self.avatar_size)

            # Set the key with no value to avoid spamming requests
            self.avatars[user["id"]] = None

        colour = self.border_col
        mute = False
        deaf = False
        bg_col = None
        fg_col = None
        tw = 0 

        if "mute" in user and user["mute"]:
            mute = True
        if "deaf" in user and user["deaf"]:
            deaf = True
        if "speaking" in user and user["speaking"] and not deaf and not mute:
            colour = self.talk_col
        if "speaking" in user and user["speaking"] and not deaf and not mute:
            bg_col = self.hili_col
            fg_col = self.text_hili_col
        else:
            bg_col = self.norm_col
            fg_col = self.text_col

        pix = None
        if user["id"] in self.avatars:
            pix = self.avatars[user["id"]]
        if not self.horizontal:
            if not self.icon_only:
                tw = self.draw_text(
                    context, user["friendlyname"],
                    pos_x,
                    pos_y,
                    fg_col,
                    bg_col,
                    avatar_size
                )
        self.draw_avatar_pix(context, pix,pos_x,pos_y,colour, avatar_size)
        if deaf:
            self.draw_deaf(context, pos_x, pos_y, avatar_size)
        elif mute:
            self.draw_mute(context, pos_x, pos_y, avatar_size)
        return tw

    def draw_text(self, context, string, pos_x, pos_y, tx_col, bg_col, avatar_size):
        """
        Draw username & background at given position
        """
        layout = self.create_pango_layout(string)
        layout.set_auto_dir(True)
        layout.set_markup(string, -1)

        layout.set_width(Pango.SCALE * self.width)
        layout.set_spacing(Pango.SCALE * 3)
        font = None
        if self.text_font:
            font = Pango.FontDescription(self.text_font)
            layout.set_font_description(font)
        (_ink_rect, logical_rect) = layout.get_pixel_extents()
        text_height = logical_rect.height
        text_width = logical_rect.width

        self.col(tx_col)
        height_offset = (avatar_size / 2) - (text_height / 2)
        text_y_offset = height_offset + self.text_baseline_adj

        if self.align_right:
            context.move_to(0, 0)
            self.col(bg_col)
            context.rectangle(
                pos_x - text_width - (self.text_pad * 2),
                pos_y + height_offset - self.text_pad,
                text_width + (self.text_pad * 4),
                text_height + (self.text_pad * 2)
            )
            context.fill()

            self.col(tx_col)
            context.move_to(
                pos_x - text_width - self.text_pad,
                pos_y + text_y_offset
            )
            PangoCairo.show_layout(self.context, layout)
        else:
            context.move_to(0, 0)
            self.col(bg_col)
            context.rectangle(
                pos_x - (self.text_pad * 2) + avatar_size,
                pos_y + height_offset - self.text_pad,
                text_width + (self.text_pad * 4),
                text_height + (self.text_pad * 2)
            )
            context.fill()

            self.col(tx_col)
            context.move_to(
                pos_x + self.text_pad+ avatar_size,
                pos_y + text_y_offset
            )
            PangoCairo.show_layout(self.context, layout)

        return text_width

    def draw_avatar_pix(self, context, pixbuf, pos_x, pos_y, border_colour, avatar_size):
        """
        Draw avatar image at given position
        """
        if not pixbuf:
            pixbuf = self.def_avatar
            if not pixbuf:
                return
        context.move_to(pos_x, pos_y)
        context.save()
        if self.round_avatar:
            context.arc(pos_x + (self.avatar_size / 2), pos_y +
                        (self.avatar_size / 2), self.avatar_size / 2, 0, 2 * math.pi)
            context.clip()

        self.set_norm_col()
        context.set_operator(cairo.OPERATOR_SOURCE)
        context.rectangle(pos_x, pos_y, avatar_size, avatar_size)
        context.fill()
        draw_img_to_rect(pixbuf, context, pos_x, pos_y,
                         avatar_size, avatar_size)
        context.restore()
        if border_colour:
            if self.round_avatar:
                context.arc(
                    pos_x + (avatar_size / 2),
                    pos_y + (avatar_size / 2),
                    avatar_size / 2,
                    0, 2 * math.pi
                )
                self.col(border_colour)
                context.stroke()
            else:
                context.rectangle(
                    pos_x, pos_y,
                    avatar_size, avatar_size
                )
                self.col(border_colour)
                context.stroke()

    def draw_mute(self, context, pos_x, pos_y, avatar_size):
        """
        Draw Mute logo
        """
        context.save()
        context.translate(pos_x, pos_y)
        context.scale(avatar_size, avatar_size)
        self.set_mute_col()
        context.save()

        # Clip Strike-through
        context.set_fill_rule(cairo.FILL_RULE_EVEN_ODD)
        context.set_line_width(0.1)
        context.move_to(0.0, 0.0)
        context.line_to(1.0, 0.0)
        context.line_to(1.0, 1.0)
        context.line_to(0.0, 1.0)
        context.line_to(0.0, 0.0)
        context.close_path()
        context.new_sub_path()
        context.arc(0.9, 0.1, 0.05, 1.25 * math.pi, 2.25 * math.pi)
        context.arc(0.1, 0.9, 0.05, .25 * math.pi, 1.25 * math.pi)
        context.close_path()
        context.clip()

        # Center
        context.set_line_width(0.07)
        context.arc(0.5, 0.3, 0.1, math.pi, 2 * math.pi)
        context.arc(0.5, 0.5, 0.1, 0, math.pi)
        context.close_path()
        context.fill()

        context.set_line_width(0.05)

        # Stand rounded
        context.arc(0.5, 0.5, 0.15, 0, 1.0 * math.pi)
        context.stroke()

        # Stand vertical
        context.move_to(0.5, 0.65)
        context.line_to(0.5, 0.75)
        context.stroke()

        # Stand horizontal
        context.move_to(0.35, 0.75)
        context.line_to(0.65, 0.75)
        context.stroke()

        context.restore()
        # Strike through
        context.arc(0.7, 0.3, 0.035, 1.25 * math.pi, 2.25 * math.pi)
        context.arc(0.3, 0.7, 0.035, .25 * math.pi, 1.25 * math.pi)
        context.close_path()
        context.fill()

        context.restore()

    def draw_deaf(self, context, pos_x, pos_y, avatar_size):
        """
        Draw deaf logo
        """
        context.save()
        context.translate(pos_x, pos_y)
        context.scale(avatar_size, avatar_size)
        self.set_mute_col()
        context.save()

        # Clip Strike-through
        context.set_fill_rule(cairo.FILL_RULE_EVEN_ODD)
        context.set_line_width(0.1)
        context.move_to(0.0, 0.0)
        context.line_to(1.0, 0.0)
        context.line_to(1.0, 1.0)
        context.line_to(0.0, 1.0)
        context.line_to(0.0, 0.0)
        context.close_path()
        context.new_sub_path()
        context.arc(0.9, 0.1, 0.05, 1.25 * math.pi, 2.25 * math.pi)
        context.arc(0.1, 0.9, 0.05, .25 * math.pi, 1.25 * math.pi)
        context.close_path()
        context.clip()

        # Top band
        context.arc(0.5, 0.5, 0.2, 1.0 * math.pi, 0)
        context.stroke()

        # Left band
        context.arc(0.28, 0.65, 0.075, 1.5 * math.pi, 0.5 * math.pi)
        context.move_to(0.3, 0.5)
        context.line_to(0.3, 0.75)
        context.stroke()

        # Right band
        context.arc(0.72, 0.65, 0.075, 0.5 * math.pi, 1.5 * math.pi)
        context.move_to(0.7, 0.5)
        context.line_to(0.7, 0.75)
        context.stroke()

        context.restore()
        # Strike through
        context.arc(0.7, 0.3, 0.035, 1.25 * math.pi, 2.25 * math.pi)
        context.arc(0.3, 0.7, 0.035, .25 * math.pi, 1.25 * math.pi)
        context.close_path()
        context.fill()

        context.restore()



#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Overview setting tab on settings window"""
import gettext
import json
import logging
import pkg_resources
from configparser import ConfigParser
import gi
import sys
from .settings import SettingsWindow

log = logging.getLogger(__name__)
t = gettext.translation('default', pkg_resources.resource_filename(
    'discover_overlay', 'locales'))
_ = t.gettext

gi.require_version("Gtk", "3.0")
# pylint: disable=wrong-import-position,wrong-import-order
from gi.repository import Gtk, Gdk, GLib  # nopep8


GUILD_DEFAULT_VALUE = "0"


class AboutSettingsWindow(Gtk.Grid):
    """Basic overview and a nicer looking landing page for Steam Deck"""

    def __init__(self, discover):
        Gtk.Grid.__init__(self)
        self.discover = discover
        self.create_gui()

    def create_gui(self):
        """
        Prepare the gui
        """
        spacing_box_1 = Gtk.Box()
        spacing_box_1.set_size_request(60, 60)
        self.attach(spacing_box_1, 1, 0, 1, 1)

        icon = Gtk.Image.new_from_icon_name("discover-overlay-tray", 256)
        icon.set_pixel_size(128)
        self.attach(icon, 1, 1, 1, 1)

        spacing_box_2 = Gtk.Box()
        spacing_box_2.set_size_request(60, 60)
        self.attach(spacing_box_2, 1, 2, 1, 1)

        blurb = Gtk.Label.new(None)
        message = "<span size=\"larger\">%s (%s)</span>\n\n%s\n\n%s (<a href=\"https://discord.gg/jRKWMuDy5V\">https://discord.gg/jRKWMuDy5V</a>) %s (<a href=\"https://github.com/trigg/Discover\">https://github.com/trigg/Discover</a>)\n\n\n\n\n\n" % (
            _("Welcome to Discover Overlay"),
            pkg_resources.get_distribution('discover_overlay').version,
            _("Discover-Overlay is a GTK3 overlay written in Python3. It can be configured to show who is currently talking on discord or it can be set to display text and images from a preconfigured channel. It is fully customisable and can be configured to display anywhere on the screen. We fully support X11 and wlroots based environments. We felt the need to make this project due to the shortcomings in support on Linux by the official discord client."),
            _("Please visit our discord"),
            _(" for support. Or open an issue on our GitHub ")
        )
        blurb.set_markup(message)
        blurb.set_line_wrap(True)
        self.attach(blurb, 1, 3, 1, 1)

        killapp = Gtk.Button.new_with_label(_("Close overlay"))
        killapp.connect("pressed", self.close_app)
        self.attach(killapp, 1, 5, 1, 1)

        self.set_column_homogeneous(True)

    def close_app(self, button):
        log.info("Quit pressed")
        sys.exit(0)

    def present_settings(self):
        self.show_all()



#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""A class to assist auto-start"""
import os
import logging
try:
    from xdg.BaseDirectory import xdg_config_home, xdg_data_home
except ModuleNotFoundError:
    from xdg import XDG_CONFIG_HOME as xdg_config_home
    from xdg import XDG_DATA_HOME as xdg_data_home

log = logging.getLogger(__name__)

class Autostart:
    """A class to assist auto-start"""

    def __init__(self, app_name):
        if not app_name.endswith(".desktop"):
            app_name = "%s.desktop" % (app_name)
        self.app_name = app_name
        self.auto_locations = [os.path.join(
            xdg_config_home, 'autostart/'), '/etc/xdg/autostart/']
        self.desktop_locations = [os.path.join(
            xdg_data_home, 'applications/'), '/usr/share/applications/']
        self.auto = self.find_auto()
        self.desktop = self.find_desktop()
        log.info("Autostart info : desktop %s auto %s",
                     self.desktop, self.auto)

    def find_auto(self):
        """Check all known locations for auto-started apps"""
        for path in self.auto_locations:
            file = os.path.join(path, self.app_name)
            if os.path.exists(file):
                return file
        return None

    def find_desktop(self):
        """Check all known locations for desktop apps"""
        for path in self.desktop_locations:
            file = os.path.join(path, self.app_name)
            if os.path.exists(file):
                return file
        return None

    def set_autostart(self, enable):
        """Set or Unset auto-start state"""
        if enable and not self.auto:
            # Enable
            directory = os.path.join(xdg_config_home, 'autostart')
            self.auto = os.path.join(directory, self.app_name)
            os.symlink(self.desktop, self.auto)
        elif not enable and self.auto:
            # Disable
            if os.path.islink(self.auto):
                os.remove(self.auto)

    def is_auto(self):
        """Check if it's already set to auto-start"""
        return True if self.auto else False



from setuptools import setup, find_packages


def readme():
    return open('README.md', 'r').read()


setup(
    name='discover-overlay',
    author='trigg',
    author_email='',
    version='0.5.2',
    description='Voice chat overlay',
    long_description=readme(),
    long_description_content_type='text/markdown',
    url='https://github.com/trigg/Discover',
    packages=find_packages(),
    include_package_data=True,
    data_files=[
        ('share/applications', [
            'discover_overlay.desktop'
        ]),
        ('share/icons/hicolor/256x256/apps',
         ['discover-overlay.png', 'discover-overlay-tray.png', 'discover-overlay-default.png']),
        ('share/icons/hicolor/scalable/apps',
         ['discover-overlay.svg', 'discover-overlay-tray.svg', 'discover-overlay-default.svg'])
    ],
    install_requires=[
        'PyGObject>=3.22',
        'websocket-client',
        'pyxdg',
        'requests',
        'python-pidfile>=3.0.0',
        'pillow',
        'python-xlib'
    ],
    entry_points={
        'console_scripts': [
            'discover-overlay = discover_overlay.discover_overlay:entrypoint',
        ]
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: X11 Applications :: GTK',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Communications :: Chat',
        'Topic :: Communications :: Conferencing',
    ],
    package_data={
        'discover_overlay': ['locales/*/LC_MESSAGES/*.mo']
    },
    keywords='discord overlay voice linux',
    license='GPLv3+',
)



#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Voice setting tab on settings window"""
import gettext
import json
import pkg_resources
from configparser import ConfigParser
import gi
from .settings import SettingsWindow
gi.require_version("Gtk", "3.0")
# pylint: disable=wrong-import-position,wrong-import-order
from gi.repository import Gtk, Gdk  # nopep8

t = gettext.translation(
    'default', pkg_resources.resource_filename('discover_overlay', 'locales'))
_ = t.gettext


def parse_guild_ids(guild_ids_str):
    """Parse the guild_ids from a str and return them in a list"""
    guild_ids = []
    for guild_id in guild_ids_str.split(","):
        guild_id = guild_id.strip()
        if guild_id != "":
            guild_ids.append(guild_id)
    return guild_ids


def guild_ids_to_string(guild_ids):
    """Put the guild ids into a comma seperated string."""
    return ", ".join(str(_id) for _id in guild_ids)


class VoiceSettingsWindow(SettingsWindow):
    """Voice setting tab on settings window"""

    def __init__(self, overlay, discover):
        SettingsWindow.__init__(self, discover)
        self.overlay = overlay
        self.set_size_request(400, 200)
        self.connect("destroy", self.close_window)
        self.connect("delete-event", self.close_window)
        self.placement_window = None
        self.align_x = None
        self.align_y = None
        self.bg_col = None
        self.fg_col = None
        self.tk_col = None
        self.mt_col = None
        self.hi_col = None
        self.bo_col = None
        self.t_hi_col = None
        self.avatar_size = None
        self.icon_spacing = None
        self.text_padding = None
        self.text_baseline_adj = None
        self.font = None
        self.square_avatar = None
        self.only_speaking = None
        self.highlight_self = None
        self.icon_only = None
        self.monitor = None
        self.vert_edge_padding = None
        self.horz_edge_padding = None
        self.floating = None
        self.order = None
        self.horizontal = None
        self.guild_ids = None
        self.overflow = None
        self.init_config()
        self.guild_filter_string = ""
        self.warned = False
        self.show_dummy = False
        self.dummy_count = 10

        self.create_gui()

    def present_settings(self):
        """
        Show tab
        """
        self.show_all()
        if not self.floating:
            self.align_x_widget.show()
            self.align_y_widget.show()
            self.align_monitor_widget.show()
            self.align_placement_widget.hide()
        else:
            self.align_x_widget.hide()
            self.align_y_widget.hide()
            self.align_monitor_widget.show()
            self.align_placement_widget.show()

    def read_config(self):
        """
        Read 'main' section of the config file
        """
        config = ConfigParser(interpolation=None)
        config.read(self.config_file)
        self.align_x = config.getboolean("main", "rightalign", fallback=False)
        self.align_y = config.getint("main", "topalign", fallback=1)
        self.bg_col = json.loads(config.get(
            "main", "bg_col", fallback="[0.0,0.0,0.0,0.5]"))
        self.fg_col = json.loads(config.get(
            "main", "fg_col", fallback="[1.0,1.0,1.0,1.0]"))
        self.t_hi_col = json.loads(config.get(
            "main", "fg_hi_col", fallback="[1.0,1.0,1.0,1.0]"))
        self.tk_col = json.loads(config.get(
            "main", "tk_col", fallback="[0.0,0.7,0.0,1.0]"))
        self.mt_col = json.loads(config.get(
            "main", "mt_col", fallback="[0.6,0.0,0.0,1.0]"))
        self.hi_col = json.loads(config.get(
            "main", "hi_col", fallback="[0.0,0.0,0.0,0.5]"))
        self.bo_col = json.loads(config.get(
            "main", "bo_col", fallback="[0.0,0.0,0.0,0.0]"))
        self.avatar_size = config.getint("main", "avatar_size", fallback=48)
        self.icon_spacing = config.getint("main", "icon_spacing", fallback=8)
        self.text_padding = config.getint("main", "text_padding", fallback=6)
        self.text_baseline_adj = config.getint(
            "main", "text_baseline_adj", fallback=0)
        self.font = config.get("main", "font", fallback=None)
        self.square_avatar = config.getboolean(
            "main", "square_avatar", fallback=True)
        self.only_speaking = config.getboolean(
            "main", "only_speaking", fallback=False)
        self.highlight_self = config.getboolean(
            "main", "highlight_self", fallback=False)
        self.icon_only = config.getboolean(
            "main", "icon_only", fallback=False)
        self.monitor = config.get("main", "monitor", fallback="None")
        self.vert_edge_padding = config.getint(
            "main", "vert_edge_padding", fallback=0)
        self.horz_edge_padding = config.getint(
            "main", "horz_edge_padding", fallback=0)
        self.floating = config.getboolean("main", "floating", fallback=False)
        self.floating_x = config.getint("main", "floating_x", fallback=0)
        self.floating_y = config.getint("main", "floating_y", fallback=0)
        self.floating_w = config.getint("main", "floating_w", fallback=400)
        self.floating_h = config.getint("main", "floating_h", fallback=400)
        self.order = config.getint("main", "order", fallback=0)
        self.autohide = config.getboolean("text", "autohide", fallback=False)
        self.horizontal = config.getboolean(
            "main", "horizontal", fallback=False)
        self.guild_ids = parse_guild_ids(
            config.get("main", "guild_ids", fallback=""))
        self.overflow = config.getint("main", "overflow", fallback = 0)

        # Pass all of our config over to the overlay
        self.overlay.set_align_x(self.align_x)
        self.overlay.set_align_y(self.align_y)
        self.overlay.set_bg(self.bg_col)
        self.overlay.set_fg(self.fg_col)
        self.overlay.set_tk(self.tk_col)
        self.overlay.set_mt(self.mt_col)
        self.overlay.set_hi(self.hi_col)
        self.overlay.set_bo(self.bo_col)
        self.overlay.set_fg_hi(self.t_hi_col)
        self.overlay.set_avatar_size(self.avatar_size)
        self.overlay.set_icon_spacing(self.icon_spacing)
        self.overlay.set_text_padding(self.text_padding)
        self.overlay.set_text_baseline_adj(self.text_baseline_adj)
        self.overlay.set_square_avatar(self.square_avatar)
        self.overlay.set_only_speaking(self.only_speaking)
        self.overlay.set_highlight_self(self.highlight_self)
        self.overlay.set_icon_only(self.icon_only)
        self.overlay.set_monitor(self.get_monitor_index(
            self.monitor), self.get_monitor_obj(self.monitor))
        self.overlay.set_vert_edge_padding(self.vert_edge_padding)
        self.overlay.set_horz_edge_padding(self.horz_edge_padding)
        self.overlay.set_order(self.order)
        self.overlay.set_hide_on_mouseover(self.autohide)
        self.overlay.set_horizontal(self.horizontal)
        self.overlay.set_guild_ids(self.guild_ids)
        self.overlay.set_overflow(self.overflow)

        self.overlay.set_floating(
            self.floating, self.floating_x, self.floating_y, self.floating_w, self.floating_h)

        if self.font:
            self.overlay.set_font(self.font)

    def save_config(self):
        """
        Write settings out to the 'main' section of the config file
        """
        config = ConfigParser(interpolation=None)
        config.read(self.config_file)
        if not config.has_section("main"):
            config.add_section("main")

        config.set("main", "rightalign", "%d" % (int(self.align_x)))
        config.set("main", "topalign", "%d" % (self.align_y))
        config.set("main", "bg_col", json.dumps(self.bg_col))
        config.set("main", "fg_col", json.dumps(self.fg_col))
        config.set("main", "tk_col", json.dumps(self.tk_col))
        config.set("main", "mt_col", json.dumps(self.mt_col))
        config.set("main", "hi_col", json.dumps(self.hi_col))
        config.set("main", "fg_hi_col", json.dumps(self.t_hi_col))
        config.set("main", "bo_col", json.dumps(self.bo_col))

        config.set("main", "avatar_size", "%d" % (self.avatar_size))
        config.set("main", "icon_spacing", "%d" % (self.icon_spacing))
        config.set("main", "text_padding", "%d" % (self.text_padding))
        config.set("main", "text_baseline_adj", "%d" %
                   (self.text_baseline_adj))
        if self.font:
            config.set("main", "font", self.font)
        config.set("main", "square_avatar", "%d" % (int(self.square_avatar)))
        config.set("main", "only_speaking", "%d" % (int(self.only_speaking)))
        config.set("main", "highlight_self", "%d" % (int(self.highlight_self)))
        config.set("main", "icon_only", "%d" % (int(self.icon_only)))
        config.set("main", "monitor", self.monitor)
        config.set("main", "vert_edge_padding", "%d" %
                   (self.vert_edge_padding))
        config.set("main", "horz_edge_padding", "%d" %
                   (self.horz_edge_padding))
        config.set("main", "floating", "%s" % (int(self.floating)))
        config.set("main", "floating_x", "%s" % (int(self.floating_x)))
        config.set("main", "floating_y", "%s" % (int(self.floating_y)))
        config.set("main", "floating_w", "%s" % (int(self.floating_w)))
        config.set("main", "floating_h", "%s" % (int(self.floating_h)))
        config.set("main", "order", "%s" % (self.order))
        config.set("main", "horizontal", "%s" % (self.horizontal))
        config.set("main", "guild_ids", "%s" %
                   guild_ids_to_string(self.guild_ids))
        config.set("main", "overflow", "%s" % (int(self.overflow)))

        with open(self.config_file, 'w') as file:
            config.write(file)

    def create_gui(self):
        """
        Prepare the gui
        """
        outer_box = Gtk.Grid()
        outer_box.set_row_spacing(64)
        outer_box.set_column_spacing(64)

        monitor_box = Gtk.Grid()
        alignment_box = Gtk.Grid(row_homogeneous=True)
        colour_box = Gtk.Grid(row_homogeneous=True)
        avatar_box = Gtk.Grid(row_homogeneous=True)
        dummy_box = Gtk.Grid(row_homogeneous=True)

        colour_box.set_row_spacing(8)
        colour_box.set_column_spacing(8)

        avatar_box.set_column_spacing(8)
        alignment_box.set_column_spacing(8)

        outer_box.attach(monitor_box, 0, 0, 1, 1)
        outer_box.attach(alignment_box, 0, 1, 1, 1)
        outer_box.attach(colour_box, 1, 0, 1, 1)
        outer_box.attach(avatar_box, 1, 1, 1, 1)
        outer_box.attach(dummy_box,0,2,2,1)

        # Autohide
        #autohide_label = Gtk.Label.new("Hide on mouseover")
        #autohide = Gtk.CheckButton.new()
        # autohide.set_active(self.autohide)
        #autohide.connect("toggled", self.change_hide_on_mouseover)

        # Font chooser
        font_label = Gtk.Label.new(_("Font"))
        font = Gtk.FontButton()
        if self.font:
            font.set_font(self.font)
        font.connect("font-set", self.change_font)
        alignment_box.attach(font_label, 0, 0, 1, 1)
        alignment_box.attach(font, 1, 0, 1, 1)

        # Colours
        bg_col = Gtk.ColorButton.new_with_rgba(
            Gdk.RGBA(self.bg_col[0], self.bg_col[1], self.bg_col[2], self.bg_col[3]))
        fg_col = Gtk.ColorButton.new_with_rgba(
            Gdk.RGBA(self.fg_col[0], self.fg_col[1], self.fg_col[2], self.fg_col[3]))
        tk_col = Gtk.ColorButton.new_with_rgba(
            Gdk.RGBA(self.tk_col[0], self.tk_col[1], self.tk_col[2], self.tk_col[3]))
        mt_col = Gtk.ColorButton.new_with_rgba(
            Gdk.RGBA(self.mt_col[0], self.mt_col[1], self.mt_col[2], self.mt_col[3]))
        hi_col = Gtk.ColorButton.new_with_rgba(
            Gdk.RGBA(
                self.hi_col[0],
                self.hi_col[1],
                self.hi_col[2],
                self.hi_col[3]))
        t_hi_col = Gtk.ColorButton.new_with_rgba(
            Gdk.RGBA(
                self.t_hi_col[0],
                self.t_hi_col[1],
                self.t_hi_col[2],
                self.t_hi_col[3]))
        bo_col = Gtk.ColorButton.new_with_rgba(
            Gdk.RGBA(
                self.bo_col[0],
                self.bo_col[1],
                self.bo_col[2],
                self.bo_col[3]
            )
        )
        bg_col.set_use_alpha(True)
        fg_col.set_use_alpha(True)
        tk_col.set_use_alpha(True)
        mt_col.set_use_alpha(True)
        hi_col.set_use_alpha(True)
        t_hi_col.set_use_alpha(True)
        bo_col.set_use_alpha(True)
        bg_col.connect("color-set", self.change_bg)
        fg_col.connect("color-set", self.change_fg)
        tk_col.connect("color-set", self.change_tk)
        mt_col.connect("color-set", self.change_mt)
        hi_col.connect("color-set", self.change_hi)
        t_hi_col.connect("color-set", self.change_t_hi)
        bo_col.connect("color-set", self.change_bo)

        text_label = Gtk.Label.new(_("Text"))
        background_label = Gtk.Label.new(_("Label"))
        talking_label = Gtk.Label.new(_("Talking"))
        idle_label = Gtk.Label.new(_("Idle"))
        border_label = Gtk.Label.new(_("Border"))
        mute_label = Gtk.Label.new(_("Mute"))

        colour_box.attach(text_label, 1, 0, 1, 1)
        colour_box.attach(background_label, 2, 0, 1, 1)
        colour_box.attach(border_label, 3, 0, 1, 1)
        colour_box.attach(talking_label, 0, 1, 1, 1)
        colour_box.attach(idle_label, 0, 2, 1, 1)
        colour_box.attach(mute_label, 0, 4, 1, 1)

        colour_box.attach(bg_col, 2, 2, 1, 1)
        colour_box.attach(hi_col, 2, 1, 1, 1)
        colour_box.attach(fg_col, 1, 2, 1, 1)
        colour_box.attach(t_hi_col, 1, 1, 1, 1)
        colour_box.attach(tk_col, 3, 1, 1, 1)
        colour_box.attach(bo_col, 3, 2, 1, 1)

        colour_box.attach(mt_col, 1, 4, 1, 1)

        # Monitor & Alignment
        align_label = Gtk.Label.new(_("Overlay Location"))

        align_type_box = Gtk.HBox()
        align_type_edge = Gtk.RadioButton.new_with_label(
            None, _("Anchor to edge"))
        align_type_floating = Gtk.RadioButton.new_with_label_from_widget(
            align_type_edge, _("Floating"))
        if self.floating:
            align_type_floating.set_active(True)
        align_type_box.add(align_type_edge)
        align_type_box.add(align_type_floating)

        monitor_store = Gtk.ListStore(str)
        display = Gdk.Display.get_default()
        if "get_n_monitors" in dir(display):
            for i in range(0, display.get_n_monitors()):
                monitor_store.append([display.get_monitor(i).get_model()])
        monitor = Gtk.ComboBox.new_with_model(monitor_store)
        monitor.set_active(self.get_monitor_index(self.monitor))
        monitor.connect("changed", self.change_monitor)
        renderer_text = Gtk.CellRendererText()
        monitor.pack_start(renderer_text, True)
        monitor.add_attribute(renderer_text, "text", 0)

        self.align_x_store = Gtk.ListStore(str)
        self.align_x_store.append([_("Left")])
        self.align_x_store.append([_("Right")])
        align_x = Gtk.ComboBox.new_with_model(self.align_x_store)
        align_x.set_active(True if self.align_x else False)
        align_x.connect("changed", self.change_align_x)
        renderer_text = Gtk.CellRendererText()
        align_x.pack_start(renderer_text, True)
        align_x.add_attribute(renderer_text, "text", 0)

        self.align_y_store = Gtk.ListStore(str)
        self.align_y_store.append([_("Top")])
        self.align_y_store.append([_("Middle")])
        self.align_y_store.append([_("Bottom")])
        align_y = Gtk.ComboBox.new_with_model(self.align_y_store)
        align_y.set_active(self.align_y)
        align_y.connect("changed", self.change_align_y)
        renderer_text = Gtk.CellRendererText()
        align_y.pack_start(renderer_text, True)
        align_y.add_attribute(renderer_text, "text", 0)

        align_placement_button = Gtk.Button.new_with_label(_("Place Window"))

        align_type_edge.connect("toggled", self.change_align_type_edge)
        align_type_floating.connect("toggled", self.change_align_type_floating)
        align_placement_button.connect("pressed", self.change_placement)

        self.align_x_widget = align_x
        self.align_y_widget = align_y
        self.align_monitor_widget = monitor
        self.align_placement_widget = align_placement_button

        monitor_box.attach(align_label, 0, 0, 2, 1)
        monitor_box.attach(align_type_box, 1, 1, 1, 1)
        monitor_box.attach(monitor, 1, 2, 1, 1)
        monitor_box.attach(align_x, 1, 3, 1, 1)
        monitor_box.attach(align_y, 1, 4, 1, 1)
        monitor_box.attach(align_placement_button, 1, 5, 1, 1)

        # Avatar size
        avatar_size_label = Gtk.Label.new(_("Avatar size"))
        avatar_size_label.set_xalign(0)
        avatar_adjustment = Gtk.Adjustment.new(
            self.avatar_size, 8, 128, 1, 1, 8)
        avatar_size = Gtk.SpinButton.new(avatar_adjustment, 0, 0)
        avatar_size.connect("value-changed", self.change_avatar_size)

        avatar_box.attach(avatar_size_label, 0, 0, 1, 1)
        avatar_box.attach(avatar_size, 1, 0, 1, 1)

        # Avatar shape
        square_avatar_label = Gtk.Label.new(_("Square Avatar"))
        square_avatar_label.set_xalign(0)
        square_avatar = Gtk.CheckButton.new()
        square_avatar.set_active(self.square_avatar)
        square_avatar.connect("toggled", self.change_square_avatar)

        avatar_box.attach(square_avatar_label, 0, 2, 1, 1)
        avatar_box.attach(square_avatar, 1, 2, 1, 1)

        # Display icon only
        icon_only_label = Gtk.Label.new(_("Display Icon Only"))
        icon_only_label.set_xalign(0)
        icon_only = Gtk.CheckButton.new()
        icon_only.set_active(self.icon_only)
        icon_only.connect("toggled", self.change_icon_only)

        avatar_box.attach(icon_only_label, 0, 1, 1, 1)
        avatar_box.attach(icon_only, 1, 1, 1, 1)

        # Display Speaker only
        only_speaking_label = Gtk.Label.new(_("Display Speakers Only"))
        only_speaking_label.set_xalign(0)
        only_speaking = Gtk.CheckButton.new()
        only_speaking.set_active(self.only_speaking)
        only_speaking.connect("toggled", self.change_only_speaking)

        avatar_box.attach(only_speaking_label, 0, 3, 1, 1)
        avatar_box.attach(only_speaking, 1, 3, 1, 1)

        # Highlight self
        highlight_self_label = Gtk.Label.new(_("Highlight Self"))
        highlight_self_label.set_xalign(0)
        highlight_self = Gtk.CheckButton.new()
        highlight_self.set_active(self.highlight_self)
        highlight_self.connect("toggled", self.change_highlight_self)

        avatar_box.attach(highlight_self_label, 0, 4, 1, 1)
        avatar_box.attach(highlight_self, 1, 4, 1, 1)

        # Order avatars
        order_label = Gtk.Label.new(_("Order Avatars By"))
        order_label.set_xalign(0)
        order_store = Gtk.ListStore(str)
        order_store.append([_("Alphabetically")])
        order_store.append([_("ID")])
        order_store.append([_("Last Spoken")])
        order = Gtk.ComboBox.new_with_model(order_store)
        order.set_active(self.order)
        order.connect("changed", self.change_order)
        renderer_text = Gtk.CellRendererText()
        order.pack_start(renderer_text, True)
        order.add_attribute(renderer_text, "text", 0)

        avatar_box.attach(order_label, 0, 5, 1, 1)
        avatar_box.attach(order, 1, 5, 1, 1)

        # Icon spacing
        icon_spacing_label = Gtk.Label.new(_("Icon Spacing"))
        icon_spacing_label.set_xalign(0)
        icon_spacing_adjustment = Gtk.Adjustment.new(
            self.icon_spacing, 0, 64, 1, 1, 0)
        icon_spacing = Gtk.SpinButton.new(icon_spacing_adjustment, 0, 0)
        icon_spacing.connect("value-changed", self.change_icon_spacing)

        alignment_box.attach(icon_spacing_label, 0, 1, 1, 1)
        alignment_box.attach(icon_spacing, 1, 1, 1, 1)

        # Text padding
        text_padding_label = Gtk.Label.new(_("Text Padding"))
        text_padding_label.set_xalign(0)
        text_padding_adjustment = Gtk.Adjustment.new(
            self.text_padding, 0, 64, 1, 1, 0)
        text_padding = Gtk.SpinButton.new(text_padding_adjustment, 0, 0)
        text_padding.connect("value-changed", self.change_text_padding)

        alignment_box.attach(text_padding_label, 0, 2, 1, 1)
        alignment_box.attach(text_padding, 1, 2, 1, 1)

        # Text Baseline Adjustment
        text_baseline_label = Gtk.Label.new(_("Text Vertical Offset"))
        text_baseline_label.set_xalign(0)
        text_baseline_adjustment = Gtk.Adjustment.new(
            self.text_baseline_adj, -32, 32, 1, 1, 0)
        text_baseline = Gtk.SpinButton.new(text_baseline_adjustment, 0, 0)
        text_baseline.connect("value-changed", self.change_text_baseline)

        alignment_box.attach(text_baseline_label, 0, 3, 1, 1)
        alignment_box.attach(text_baseline, 1, 3, 1, 1)

        # Edge padding
        vert_edge_padding_label = Gtk.Label.new(_("Vertical Edge Padding"))
        vert_edge_padding_label.set_xalign(0)
        vert_edge_padding_adjustment = Gtk.Adjustment.new(
            self.vert_edge_padding, 0, 1000, 1, 1, 0)
        vert_edge_padding = Gtk.SpinButton.new(
            vert_edge_padding_adjustment, 0, 0)
        vert_edge_padding.connect(
            "value-changed", self.change_vert_edge_padding)

        alignment_box.attach(vert_edge_padding_label, 0, 4, 1, 1)
        alignment_box.attach(vert_edge_padding, 1, 4, 1, 1)

        horz_edge_padding_label = Gtk.Label.new(_("Horizontal Edge Padding"))
        horz_edge_padding_adjustment = Gtk.Adjustment.new(
            self.horz_edge_padding, 0, 1000, 1, 1, 0)
        horz_edge_padding = Gtk.SpinButton.new(
            horz_edge_padding_adjustment, 0, 0)
        horz_edge_padding.connect(
            "value-changed", self.change_horz_edge_padding)

        alignment_box.attach(horz_edge_padding_label, 0, 5, 1, 1)
        alignment_box.attach(horz_edge_padding, 1, 5, 1, 1)

        # Display icon horizontally
        horizontal_label = Gtk.Label.new(_("Display Horizontally"))
        horizontal_label.set_xalign(0)
        horizontal = Gtk.CheckButton.new()
        horizontal.set_active(self.horizontal)
        horizontal.connect("toggled", self.change_horizontal)

        alignment_box.attach(horizontal_label, 0, 6, 1, 1)
        alignment_box.attach(horizontal, 1, 6, 1, 1)

        # Overflow
        overflow_label = Gtk.Label.new(_("Overflow style"))
        overflow_label.set_xalign(0)
        overflow_store = Gtk.ListStore(str)
        overflow_store.append([_("None")])
        overflow_store.append([_("Wrap")])
        overflow_store.append([_("Shrink")])
        overflow = Gtk.ComboBox.new_with_model(overflow_store)
        overflow.set_active(self.overflow)
        overflow.connect("changed", self.change_overflow)
        renderer_text = Gtk.CellRendererText()
        overflow.pack_start(renderer_text, True)
        overflow.add_attribute(renderer_text, "text", 0)

        avatar_box.attach(overflow_label, 0, 6, 1, 1)
        avatar_box.attach(overflow, 1, 6, 1, 1)

        # use dummy
        dummy_label = Gtk.Label.new(_("Show test content"))
        dummy_label.set_xalign(0)
        dummy = Gtk.CheckButton.new()
        dummy.set_active(self.show_dummy)
        dummy.connect("toggled", self.change_dummy_data)

        dummy_box.attach(dummy_label, 0, 0, 1, 1)
        dummy_box.attach(dummy, 1, 0, 1, 1)

        # Dummy count
        dummy_count_label = Gtk.Label.new(_("Dummy count"))
        dummy_count_label.set_xalign(0)
        dummy_count_adjustment = Gtk.Adjustment.new(
            self.dummy_count, 0, 100, 1, 1, 0)
        dummy_count = Gtk.SpinButton.new(
            dummy_count_adjustment, 0, 0)
        dummy_count.connect(
            "value-changed", self.change_dummy_count)

        dummy_box.attach(dummy_count_label, 2, 0, 1, 1)
        dummy_box.attach(dummy_count, 3, 0, 1, 1)

        self.add(outer_box)

        self.set_orientated_names()

    def change_font(self, button):
        """
        Font settings changed
        """
        font = button.get_font()
        self.overlay.set_font(font)

        self.font = font
        self.save_config()

    def change_bg(self, button):
        """
        Background colour changed
        """
        colour = button.get_rgba()
        colour = [colour.red, colour.green, colour.blue, colour.alpha]
        self.overlay.set_bg(colour)

        self.bg_col = colour
        self.save_config()

    def change_fg(self, button):
        """
        Text colour changed
        """
        colour = button.get_rgba()
        colour = [colour.red, colour.green, colour.blue, colour.alpha]
        self.overlay.set_fg(colour)

        self.fg_col = colour
        self.save_config()

    def change_tk(self, button):
        """
        Talking colour changed
        """
        colour = button.get_rgba()
        colour = [colour.red, colour.green, colour.blue, colour.alpha]
        self.overlay.set_tk(colour)

        self.tk_col = colour
        self.save_config()

    def change_mt(self, button):
        """
        Mute colour changed
        """
        colour = button.get_rgba()
        colour = [colour.red, colour.green, colour.blue, colour.alpha]
        self.overlay.set_mt(colour)

        self.mt_col = colour
        self.save_config()

    def change_hi(self, button):
        """
        Speaking background colour changed
        """
        colour = button.get_rgba()
        colour = [colour.red, colour.green, colour.blue, colour.alpha]
        self.overlay.set_hi(colour)

        self.hi_col = colour
        self.save_config()

    def change_t_hi(self, button):
        """
        Speaking background colour changed
        """
        colour = button.get_rgba()
        colour = [colour.red, colour.green, colour.blue, colour.alpha]
        self.overlay.set_fg_hi(colour)

        self.t_hi_col = colour
        self.save_config()

    def change_bo(self, button):
        """
        Idle border colour changed
        """
        colour = button.get_rgba()
        colour = [colour.red, colour.green, colour.blue, colour.alpha]
        self.overlay.set_bo(colour)

        self.bo_col = colour
        self.save_config()

    def change_avatar_size(self, button):
        """
        Avatar size setting changed
        """
        self.overlay.set_avatar_size(button.get_value())

        self.avatar_size = button.get_value()
        self.save_config()

    def change_icon_spacing(self, button):
        """
        Inter-icon spacing changed
        """
        self.overlay.set_icon_spacing(button.get_value())

        self.icon_spacing = int(button.get_value())
        self.save_config()

    def change_text_padding(self, button):
        """
        Text padding changed
        """
        self.overlay.set_text_padding(button.get_value())

        self.text_padding = button.get_value()
        self.save_config()

    def change_text_baseline(self, button):
        """
        Text baseline changed
        """
        self.overlay.set_text_baseline_adj(button.get_value())

        self.text_baseline_adj = button.get_value()
        self.save_config()

    def change_vert_edge_padding(self, button):
        """
        Vertical padding changed
        """
        self.overlay.set_vert_edge_padding(button.get_value())

        self.vert_edge_padding = button.get_value()
        self.save_config()

    def change_horz_edge_padding(self, button):
        """
        Horizontal padding changed
        """
        self.overlay.set_horz_edge_padding(button.get_value())

        self.horz_edge_padding = button.get_value()
        self.save_config()

    def change_square_avatar(self, button):
        """
        Square avatar setting changed
        """
        self.overlay.set_square_avatar(button.get_active())

        self.square_avatar = button.get_active()
        self.save_config()

    def change_only_speaking(self, button):
        """
        Show only speaking users setting changed
        """
        self.overlay.set_only_speaking(button.get_active())

        self.only_speaking = button.get_active()
        self.save_config()

    def change_highlight_self(self, button):
        """
        Highlight self setting changed
        """
        self.overlay.set_highlight_self(button.get_active())

        self.highlight_self = button.get_active()
        self.save_config()

    def change_icon_only(self, button):
        """
        Icon only setting changed
        """
        self.overlay.set_icon_only(button.get_active())

        self.icon_only = button.get_active()
        self.save_config()

    def change_order(self, button):
        """
        Order user setting changed
        """
        self.overlay.set_order(button.get_active())

        self.order = button.get_active()
        self.save_config()

    def change_horizontal(self, button):
        """
        Horizontal layout setting changed
        """
        self.overlay.set_horizontal(button.get_active())

        self.horizontal = button.get_active()
        self.save_config()
        self.set_orientated_names()

    def change_dummy_data(self, button):
        self.overlay.set_show_dummy(button.get_active())
        self.show_dummy = button.get_active()

    def change_dummy_count(self, button):
        self.overlay.set_dummy_count(int(button.get_value()))
        self.dummy_count = button.get_value()

    def change_overflow(self, button):
        self.overlay.set_overflow(button.get_active())
        self.overflow = button.get_active()
        self.save_config()

    def on_guild_selection_changed(self, tree, number, selection):
        model, treeiter = tree.get_selection().get_selected()
        if treeiter is not None:
            model[treeiter][0] = not model[treeiter][0]
            if model[treeiter][0]:
                self.add_guild(model[treeiter][3])
            else:
                self.remove_guild(model[treeiter][3])

    def add_guild(self, guild):
        self.guild_ids.append(guild)
        self.overlay.set_guild_ids(self.guild_ids)
        self.save_config()

    def remove_guild(self, guild):
        self.guild_ids.remove(guild)
        self.overlay.set_guild_ids(self.guild_ids)
        self.save_config()

    def change_guild_ids(self, input):
        """
        Guild IDs replaced
        """
        self.guild_ids = parse_guild_ids(input.get_text())
        self.overlay.set_guild_ids(self.guild_ids)
        self.save_config()

    def guild_filter_changed(self, entry):
        """
        Change the filter string for guilds
        """
        self.guild_filter_string = entry.get_text()
        self.guild_ids_filter.refilter()

    def guild_filter_func(self, model, iter, data):
        """
        Decide if a guild is shown in the list of guilds
        """
        if self.guild_filter_string in model[iter][2]:
            return True
        return False

    def set_orientated_names(self):
        i = self.align_x_store.get_iter_first()
        i2 = self.align_y_store.get_iter_first()
        if self.horizontal:
            self.align_x_store.set_value(i, 0, _("Top"))
            i = self.align_x_store.iter_next(i)
            self.align_x_store.set_value(i, 0, _("Bottom"))

            self.align_y_store.set_value(i2, 0, _("Left"))
            i2 = self.align_y_store.iter_next(i2)
            self.align_y_store.set_value(i2, 0, _("Middle"))
            i2 = self.align_y_store.iter_next(i2)
            self.align_y_store.set_value(i2, 0, _("Right"))
        else:
            self.align_x_store.set_value(i, 0, _("Left"))
            i = self.align_x_store.iter_next(i)
            self.align_x_store.set_value(i, 0, _("Right"))

            self.align_y_store.set_value(i2, 0, _("Top"))
            i2 = self.align_y_store.iter_next(i2)
            self.align_y_store.set_value(i2, 0, _("Middle"))
            i2 = self.align_y_store.iter_next(i2)
            self.align_y_store.set_value(i2, 0, _("Bottom"))
